/*
 * libssc: Library to expose Qualcomm Sensor Core sensors
 * Copyright (C) 2022-2025 Dylan Van Assche
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
syntax = "proto2";

/*
 * Accelerometer Sensor specific response data.
 * @acceleration: acceleration per axis in m/s²: X-axis [0], Y-axis [1], Z-axis [2].
 * @accuracy: how reliable the value is [0, 3]. Higher is more accurate, 0 is unreliable.
 */
message SscAccelerometerResponse {
	repeated float acceleration = 1;
	required int32 accuracy = 2;
}
