/*
 * libssc: Library to expose Qualcomm Sensor Core sensors
 * Copyright (C) 2022-2025 Dylan Van Assche
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
syntax = "proto2";
import "ssc-common.proto";

/* 
 * SUID Sensor specific request data.
 * @data_type: retrieve all sensors exposing the specified data type.
 * An empty value will return all sensors.
 * @enable_updates: setting this to true will enable updates 
 * for the specified data_type.
 * @only_default_values: setting this to true will only return 
 * the default sensor for the data_type, even if multiple sensors expose it.
 */
message SscSuidRequest {
	required string data_type = 1;
	optional bool enable_updates = 2;
	optional bool only_default_values = 3;
}

/*
 * SUID Sensor specific response data.
 * @data_type: the data_type for which the list of UIDs match.
 * @uid: list of matching UIDs.
 */
message SscSuidResponse {
	required string data_type = 1;
	repeated SscUid uid = 2;
}
